@@ -127,12 +127,14 @@ class EstimatePi:
 
         return error, pi_estimation
 
-def functional_std_approximation(f, variables, size=1000):
+
+def functional_std_approximation(f, variables: List[Tuple[float]], size=1000):
     """
     Give a functional approximation of the standard deviation after performing a function on the inputs.
     """
     return np.std(f(*[np.random.normal(m, s, size=size) for m, s in variables]))
 
+
 def calculus_std_approximation(f, variables: List[Tuple[float]]):
     """
     Give an approximation of the standard deviation after performing a function on the inputs, using calculus.
@@ -149,3 +151,53 @@ def calculus_std_approximation(f, variables: List[Tuple[float]]):
     relative_uncertainties = [(derivative * x[1])**2 for derivative, x in zip(derivatives, variables)]
     uncertainty = np.sum(relative_uncertainties)**.5
     return uncertainty
+
+
+def unpack_variables(variables: List[Tuple]):
+    """
+    Unpack variables so they can be used for std_approximation.
+    """
+    shapes = [np.array(variable[0]).shape for variable in variables] + [(1,)]
+    cast_shape = max(variable[0], key=len)
+    return [(cast_to(variable[0], largest), cast_to(variable[1], cast_shape)) 
+            for variable in variables], cast_shape
+
+
+def cast_to(variable: List[Tuple], shape: Tuple[int]):
+    v_shape = np.array(variable).shape
+    cast_shape = []
+    orig_shape = []
+    i = 0
+    for dim in shape:
+        if i < len(v_shape) and dim == v_shape[i]:
+            i += 1
+            cast_shape += [1]
+            orig_shape += [v_shape[i]]
+            continue
+        cast_shape += [dim]
+        orig_shape += [i]
+    return np.array(variable).reshape(orig_shape)*np.ones(cast_shape)
+
+
+def std_approximation(f, variables: List[Tuple], approx_type: str = 'functional', *args, **kwargs):
+    unpacked, cast_shape = unpack_variables(variables)
+
+    if approx_type == 'functional':
+        approx_f = functional_std_approximation
+    elif approx_type == 'calculus':
+        approx_f = calculus_std_approximation
+    else:
+        print("This type of approximation is not supported")
+
+    std = []
+    for i in range(cast_shape[0]):
+        var_slice = [(var[0][i], var[1][i]) for var in unpacked]
+        std += [approx_f(f, var_slice, *args, **kwargs)]
+    std = np.array(std)
+    return np.squeeze(std)
+
+
+
+
+
+
